import {
  ConflictException,
  ForbiddenException,
  forwardRef,
  Inject,
  Injectable,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Equipo } from '../equipo/entity/equipo.entity';
import { EquipoPrograma } from './entity/equipo-programa.entity';
import { Operador } from '../operador/entity/operador.entity';
import { Programa } from '../institucion-programa/entity/programa.entity';
import { FullInformacionEquipoProgramaView } from './entity/views/full-informacion-equipo-programa.view';
import { InformacionEquipoProgramaView } from './entity/views/informacion-equipo-programa.view';
import { EquipoService } from '../equipo/equipo.service';
import { InstitucionProgramaService } from '../institucion-programa/institucion-programa.service';

@Injectable()
export class EquipoProgramaService {
  constructor(
    @InjectRepository(EquipoPrograma)
    private repository: Repository<EquipoPrograma>,
    @InjectRepository(FullInformacionEquipoProgramaView)
    private fullInformacionEquipoProgramaView: Repository<FullInformacionEquipoProgramaView>,
    @InjectRepository(InformacionEquipoProgramaView)
    private informacionEquipoProgramaView: Repository<InformacionEquipoProgramaView>,
    @Inject(forwardRef(() => EquipoService))
    private equipoService: EquipoService,
    private institucionProgramaService: InstitucionProgramaService,
  ) {}

  async asignar(
    operador: Operador,
    id_equipo: number,
    id_programa: number,
  ): Promise<{ message: string }> {
    const equipo = await this.equipoService.findById(id_equipo);
    const programa = await this.institucionProgramaService.findProgramaById(
      id_programa,
    );

    // Validamos que el programa seleccionado no sea "Sin software"
    if (programa.id_programa === 1)
      throw new ConflictException(
        'No se puede asignar manualmente sin software.',
      );
    // Validamos que el equipo pertenezca a la institución del operador
    if (
      operador.institucion.id_institucion !=
      equipo.carrito.modulo.institucion.id_institucion
    )
      throw new ForbiddenException(
        'No puedes modificar la información este equipo porque no pertenece a tu institución.',
      );
    // Buscamos un registro con los datos mandados del client
    return this.findByEquipoPrograma(equipo, programa)
      .then((existeEquipoPrograma) => {
        // Si encontramos uno sacamos error
        if (existeEquipoPrograma)
          throw new ConflictException(
            'Este software ya fue asignado a este equipo.',
          );
        return this.create(equipo, programa);
      })
      .then((_) => ({
        message: 'Se asignó correctamente el software a este equipo.',
      }));
  }

  create(equipo: Equipo, programa: Programa): Promise<EquipoPrograma> {
    // Verificamos si tiene programa "Sin software"
    return this.findByEquipoPrograma(equipo, { id_programa: 1 }).then(
      async (existeEquipoSinPrograma) => {
        // Si lo tiene lo eliminamos
        if (existeEquipoSinPrograma)
          await this.repository.remove(existeEquipoSinPrograma);
        // Creamos y guardamos un registro
        return this.repository.save(
          this.repository.create({ equipo, programa }),
        );
      },
    );
  }

  createSinPrograma(equipo: Equipo): Promise<EquipoPrograma> {
    return this.repository.save(this.repository.create({ equipo }));
  }

  async delete(
    operador: Operador,
    id_equipo_programa: number,
  ): Promise<{ message: string }> {
    const equipoPrograma = await this.findById(id_equipo_programa);

    // Validamos que el equipo pertenezca a la institución del operador
    if (
      operador.institucion.id_institucion !=
      equipoPrograma.equipo.carrito.modulo.institucion.id_institucion
    )
      throw new ForbiddenException(
        'No puedes modificar la información este equipo porque no pertenece a tu institución.',
      );
    // No se puede eliminar "Sin programa" manualmente
    if (equipoPrograma.programa.id_programa === 1)
      throw new ConflictException(
        'No se puede eliminar manualmente sin software.',
      );
    // Buscamos cuantos programas tiene el equipo
    return this.repository
      .count({ where: { equipo: equipoPrograma.equipo } })
      .then(async (n) => {
        // Asignar "Sin software" si es que solo tiene 1, el que se va a eliminar
        if (n === 1) await this.createSinPrograma(equipoPrograma.equipo);
        // Eliminamos el registro
        return this.repository.remove(equipoPrograma);
      })
      .then((_) => ({
        message: 'Se eliminó correctamente el software de este equipo.',
      }));
  }

  findByEquipoPrograma(
    equipo: Equipo,
    programa: Programa | Partial<Programa>,
  ): Promise<EquipoPrograma> {
    return this.repository.findOne({ where: { equipo, programa } });
  }

  findById(id_equipo_programa: number): Promise<EquipoPrograma> {
    return this.informacionEquipoProgramaView
      .findOne({ where: { id_equipo_programa } })
      .then((infoEquipoPrograma) => {
        if (!infoEquipoPrograma)
          throw new ConflictException('No existe este id equipo programa.');
        return this.repository.create({
          id_equipo_programa: infoEquipoPrograma.id_equipo_programa,
          equipo: {
            id_equipo: infoEquipoPrograma.id_equipo,
            carrito: {
              id_carrito: infoEquipoPrograma.id_carrito,
              modulo: {
                id_modulo: infoEquipoPrograma.id_modulo,
                institucion: {
                  id_institucion: infoEquipoPrograma.id_institucion,
                },
              },
              tipoCarrito: {
                id_tipo_carrito: infoEquipoPrograma.id_tipo_carrito,
              },
            },
          },
          programa: { id_programa: infoEquipoPrograma.id_programa },
        });
      });
  }

  findFullInfoAllByIdEquipo(id_equipo: number): Promise<EquipoPrograma[]> {
    return this.fullInformacionEquipoProgramaView
      .find({ where: { id_equipo } })
      .then((infoEquipoProgramas) => {
        const equipoProgramas: EquipoPrograma[] = [];

        for (let i = 0; i < infoEquipoProgramas.length; i++)
          equipoProgramas.push(
            this.repository.create({
              id_equipo_programa: infoEquipoProgramas[i].id_equipo_programa,
              programa: {
                id_programa: infoEquipoProgramas[i].id_programa,
                programa: infoEquipoProgramas[i].programa,
              },
            }),
          );
        return equipoProgramas;
      });
  }
}
